#region usings
using System;
using System.ComponentModel.Composition;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using System.Linq;

using VVVV.Core.Logging;
#endregion usings

namespace VVVV.Nodes
{

	public class RemoveSlice<T> : IPluginEvaluate
	{
		#region fields & pins	
		[Input("Input", BinSize =  1, BinName = "Bin Size")]
		ISpread<ISpread<T>> FInput;
		
		[Input("Index")]
		ISpread<int> FIndex;

		[Output("Output")]
		ISpread<ISpread<T>> FOutput;
		#endregion fields & pins


		
		//called when data for any output pin is requested
		public void Evaluate(int SpreadMax)
		{
			FOutput.AssignFrom(FInput);
			foreach (int i in FIndex.OrderByDescending(x => x))
				FOutput.RemoveAt(i);
		}
	}
	#region PluginInfo
	[PluginInfo(Name = "RemoveSlice",
	            Category = "Spreads",
	            Help = "RemoveSlice",
	            Tags = "",
	            Author = "woei")]
	#endregion PluginInfo
	public class RemoveSliceSpreads : RemoveSlice<double> {}
	
	#region PluginInfo
	[PluginInfo(Name = "RemoveSlice",
	            Category = "String",
	            Help = "RemoveSlice",
	            Tags = "",
	            Author = "woei")]
	#endregion PluginInfo
	public class RemoveSliceString : RemoveSlice<string> {}
	
	#region PluginInfo
	[PluginInfo(Name = "RemoveSlice",
	            Category = "Color",
	            Help = "RemoveSlice",
	            Tags = "split",
	            Author = "woei")]
	#endregion PluginInfo
	public class RemoveSliceColor : RemoveSlice<RGBAColor> {}
	
	#region PluginInfo
	[PluginInfo(Name = "RemoveSlice",
	            Category = "Transform",
	            Help = "RemoveSlice",
	            Tags = "",
	            Author = "woei")]
	#endregion PluginInfo
	public class RemoveSliceTransform : RemoveSlice<Matrix4x4> {}
	
	#region PluginInfo
	[PluginInfo(Name = "RemoveSlice",
	            Category = "Enumerations",
	            Help = "RemoveSlice",
	            Tags = "",
	            Author = "woei")]
	#endregion PluginInfo
	public class RemoveSliceEnum : RemoveSlice<EnumEntry> {}
}
